@@ -1,17 +1,13 @@
 #ifndef RESPONSE_H
 #define RESPONSE_H
 
-#include <Python.h>
-#include <sys/uio.h>
-#include <inttypes.h>
+#include "meinheld.h"
 
-#include "server.h"
 #include "client.h"
 #include "time_cache.h"
 
 typedef struct iovec iovec_t;
 
-
 typedef struct {
     int fd;
     iovec_t *iov;
@@ -20,6 +16,8 @@ typedef struct {
     uint32_t total;
     uint32_t total_size;
     uint8_t sended;
+    PyObject *temp1; //keep origin pointer
+    PyObject *temp2; //keep origin pointer
 } write_bucket;
 
 
@@ -34,37 +32,33 @@ typedef struct {
 
 } FileWrapperObject;
 
+typedef enum {
+    STATUS_OK = 0,
+    STATUS_SUSPEND,
+    STATUS_ERROR 
+} response_status;
+
 extern PyTypeObject ResponseObjectType;
 extern PyTypeObject FileWrapperType;
-
 extern ResponseObject *start_response;
 
-inline PyObject * 
-create_start_response(client_t *cli);
+PyObject* create_start_response(client_t *cli);
 
-inline PyObject * 
-file_wrapper(PyObject *self, PyObject *args);
+PyObject* file_wrapper(PyObject *self, PyObject *args);
 
-inline int 
-CheckFileWrapper(PyObject *obj);
+int CheckFileWrapper(PyObject *obj);
 
-inline int
-response_start(client_t *client);
+response_status response_start(client_t *client);
 
-inline int 
-process_body(client_t *client);
+response_status process_body(client_t *client);
 
-inline void 
-close_response(client_t *client);
+response_status close_response(client_t *client);
 
-inline void
-setup_start_response(void);
+void setup_start_response(void);
 
-inline void
-clear_start_response(void);
+void clear_start_response(void);
 
-inline void
-send_error_page(client_t *client);
+void send_error_page(client_t *client);
 
 
 #endif
